import { basicSetup } from 'codemirror';
import { EditorView } from '@codemirror/view';
import { StateEffect } from '@codemirror/state';
import { ViewUpdate } from '@codemirror/view';
import { formatKeymap } from './extensions/tab';
import EventEmitter from 'eventemitter3';
export type CodeEditor = EditorView & {
  emitter?: EventEmitter;
};
let editor: CodeEditor = null;

export type EditorOptions = {
  extensions?: any[];
  hasBasicSetup?: boolean;
  onChange?: (content: string) => void;
};
/**
 * 创建单例
 * @param el
 * @returns
 */
const createEditorInstance = (el?: HTMLDivElement, opts?: EditorOptions) => {
  if (editor && el) {
    el.appendChild(editor.dom);
    return editor;
  } else if (editor) {
    return editor;
  }
  const extensions = opts?.extensions || [];
  extensions.push(formatKeymap);
  const hasBaseicSetup = opts?.hasBasicSetup ?? true;
  if (hasBaseicSetup) {
    extensions.unshift(basicSetup);
  }
  const emitter = new EventEmitter();
  createOnChangeListener(emitter, opts.onChange).appendTo(extensions);
  editor = new EditorView({
    extensions: extensions,
    parent: el || document.body,
  });
  editor.emitter = emitter;
  editor.dom.style.height = '100%';
  return editor as CodeEditor;
};

/**
 * 每次都创建新的实例
 * @param el
 * @returns
 */
export const createEditor = (el: HTMLDivElement, opts?: EditorOptions) => {
  const extensions = opts?.extensions || [];
  const hasBaseicSetup = opts?.hasBasicSetup ?? true;
  if (hasBaseicSetup) {
    extensions.unshift(basicSetup);
  }
  extensions.push(formatKeymap);
  const emitter = new EventEmitter();
  createOnChangeListener(emitter, opts.onChange).appendTo(extensions);
  const editor = new EditorView({
    extensions,
    parent: el || document.body,
  }) as CodeEditor;

  editor.emitter = emitter;
  editor.dom.style.height = '100%';

  return editor as CodeEditor;
};
export const getEditor = () => editor;

export { editor, createEditorInstance };

export const createOnChangeListener = (emitter: EventEmitter, callback?: (content: string) => void) => {
  const listener = EditorView.updateListener.of((update: ViewUpdate) => {
    if (update.docChanged) {
      const editor = update.view;
      if (callback) {
        callback(editor.state.doc.toString());
      }
      // 触发自定义事件
      emitter.emit('change', editor.state.doc.toString());
    }
  });
  // 返回监听器配置，而不是直接应用它
  return {
    extension: listener,
    appendTo: (ext: any[]) => {
      ext.push(listener);
    },
  };
};
