import { EditorView } from '@codemirror/view';
import { CodeEditor } from './editor.base';
type ChainOpts = {
  editor?: CodeEditor;
};
export class Chain {
  editor: CodeEditor | EditorView;
  constructor(opts?: ChainOpts) {
    this.editor = opts?.editor;
  }
  getEditor() {
    return this.editor;
  }
  getContent() {
    return this.editor?.state.doc.toString() || '';
  }
  setContent(content: string) {
    if (this.editor) {
      this.editor.dispatch({
        changes: { from: 0, to: this.editor.state.doc.length, insert: content },
      });
    }
    return this;
  }
  setEditor(editor: EditorView) {
    this.editor = editor;
    return this;
  }
  clearEditor() {
    if (this.editor) {
      this.editor.dispatch({
        changes: { from: 0, to: this.editor.state.doc.length, insert: '' },
      });
    }
    return this;
  }
  destroy() {
    if (this.editor) {
      this.editor.destroy();
      this.editor = null;
    }
    return this;
  }
  static create(opts?: ChainOpts) {
    return new Chain(opts);
  }
  setOnChange(callback: (content: string) => void) {
    if (this.editor) {
      const editor = this.editor as CodeEditor;
      if (editor.emitter) {
        editor.emitter.on('change', callback);
        return () => {
          editor.emitter.off('change', callback);
        };
      }
    }
    return () => {};
  }
}
